unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqldb, db, sqlite3conn, FileUtil, Forms, Controls,
  Graphics, Dialogs, DBGrids, ExtCtrls, StdCtrls, PairSplitter, ZConnection,
  ZDataset;

type

  { TForm1 }

  TForm1 = class(TForm)
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    DataSource3: TDataSource;
    DataSource4: TDataSource;
    DBGrid1: TDBGrid;
    DBGrid2: TDBGrid;
    DBGrid3: TDBGrid;
    DBGrid4: TDBGrid;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Connection1: TSQLite3Connection;
    Panel4: TPanel;
    Query1: TSQLQuery;
    Connection3: TSQLite3Connection;
    Query3: TSQLQuery;
    Transaction3: TSQLTransaction;
    Transaction1: TSQLTransaction;
    Connection2: TZConnection;
    Query2: TZQuery;
    Connection4: TZConnection;
    Query4: TZQuery;
    procedure FormCreate(Sender: TObject);
  private
    FConnection: TComponent;
    FGrid: TDBGrid;
    FLbl: TLabel;
    FQuery: TDataSet;
    procedure ClearDatabase;
    procedure ConnectDatabase;
    procedure CreateDatabase;
    procedure Execute(SQLText: String);
    procedure SetFillTime(AValue: QWord);
    procedure SetGrid(AValue: TDBGrid);
    procedure SetQuery(AValue: TDataSet);
    procedure Test(aGrid: TDBGrid);
    procedure TestFill;
    procedure TestFill1;
    procedure TestFill2;
  public
    property Connection: TComponent read FConnection write FConnection;
    property FillTime: QWord write SetFillTime;
    property Grid: TDBGrid read FGrid write SetGrid;
    property Lbl: TLabel read FLbl write FLbl;
    property Query: TDataSet read FQuery write SetQuery;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  ProjectPath: String;
begin
  ProjectPath := GetCurrentDir + PathDelim;

  Connection1.DatabaseName := ProjectPath + 'test1.sql';
  Connection1.Connected := True;
  Query1.PacketRecords := 1000;

  Connection2.Database := ProjectPath + 'test2.sql';
  Connection2.LibraryLocation := ProjectPath + 'sqlite3.dll';
  Connection2.Protocol := 'sqlite-3';
  Connection2.Connected := True;

  Connection3.DatabaseName := ProjectPath + 'test3.sql';
  Connection3.Connected := True;
  Query3.PacketRecords := 1000;

  Connection4.Database := ProjectPath + 'test4.sql';
  Connection4.LibraryLocation := ProjectPath + 'sqlite3.dll';
  Connection4.Protocol := 'sqlite-3';
  Connection4.Connected := True;

  Test(DBGrid1);
  Test(DBGrid2);
  Test(DBGrid3);
  Test(DBGrid4);

end;

procedure TForm1.ClearDatabase;
begin
  Execute('DROP TABLE IF EXISTS foo;');
end;

procedure TForm1.ConnectDatabase;
begin
  if Query is TZQuery then
  begin
    TZQuery(Query).SQL.Text := 'SELECT * FROM foo;';
    TZQuery(Query).Open;
  end
  else if Query is TSQLQuery then
  begin
    TSQLQuery(Query).SQL.Text := 'SELECT * FROM foo;';
    TSQLQuery(Query).Open;
  end
  else
    raise Exception.Create('TForm1.ConnectDatabase: Unknown Query');
end;

procedure TForm1.CreateDatabase;
begin
  Execute(
    'CREATE TABLE IF NOT EXISTS foo (' +
    'id INTEGER PRIMARY KEY AUTOINCREMENT, ' +
    'str1 VARCHAR(100), ' +
    'str2 VARCHAR(100), ' +
    'str3 VARCHAR(100));');
end;

procedure TForm1.Execute(SQLText: String);
begin
  if Connection is TZConnection then
    TZConnection(Connection).ExecuteDirect(SQLText)
  else if Connection is TSQLite3Connection then
    TSQLite3Connection(Connection).ExecuteDirect(SQLText)
  else
    raise Exception.Create('TForm1.ConnectionExecute: Unknown Connection');
end;

procedure TForm1.SetFillTime(AValue: QWord);
begin
  Lbl.Caption := IntToStr(AValue);
end;

procedure TForm1.SetGrid(AValue: TDBGrid);
begin
  if FGrid = AValue then Exit;
  FGrid := AValue;
  if Assigned(FGrid.Parent) and (FGrid.Parent.Controls[0] is TLabel) then
    Lbl := TLabel(FGrid.Parent.Controls[0])
  else
    raise Exception.Create('TForm1.SetGrid Parent is not TPanel or TPanel first Control ' +
                           'isn''t a TLabel: ' + Grid.Name);
end;

procedure TForm1.SetQuery(AValue: TDataSet);
begin
  if FQuery = AValue then Exit;
  FQuery := AValue;
  if Query is TZQuery then
    Connection := TZQuery(Query).Connection
  else if Query is TSQLQuery then
    Connection := TSQLQuery(Query).DataBase
  else
    raise Exception.Create('TForm1.SetQuery: Unknown Query');
end;

procedure TForm1.Test(aGrid: TDBGrid);
begin
  Grid := aGrid;
  if Assigned(aGrid.DataSource) and Assigned(aGrid.DataSource.DataSet) then
    Query := aGrid.DataSource.DataSet
  else
    raise Exception.Create('TForm1.Test: No linked DataSource or Dataset for grid ' + aGrid.Name);
  ClearDatabase;
  CreateDatabase;
  ConnectDatabase;
  TestFill;
  ConnectDatabase;
end;

procedure TForm1.TestFill;
var
  Cnt: QWord;
begin
  Cnt := GetTickCount64;
  if Connection = Connection1 then TestFill1
  else if Connection = Connection2 then TestFill1
  else if Connection = Connection3 then TestFill2
  else if Connection = Connection4 then TestFill2;
  FillTime := GetTickCount64 - Cnt;
end;

procedure TForm1.TestFill1;
var
  i: Integer;
begin
  for i := 0 to 99 do
    Execute('INSERT INTO foo (str1, str2, str3) VALUES (' +
      '''first entry ' + IntToStr(i) + ''', ' +
      '''second entry ' + IntToStr(i) + ''', ' +
      '''third entry ' + IntToStr(i) + ''');');
  if Query is TSQLQuery then
    TSQLite3Connection(TSQLQuery(Query).DataBase).Transaction.Commit;
end;

procedure TForm1.TestFill2;
var
  i: Integer;
begin
  if Query is TZQuery then
    for i := 0 to 99 do
    begin
      TZQuery(Query).Append;
      TZQuery(Query).FieldByName('str1').AsString := 'first entry ' + IntToStr(i);
      TZQuery(Query).FieldByName('str2').AsString := 'second entry ' + IntToStr(i);
      TZQuery(Query).FieldByName('str3').AsString := 'third entry ' + IntToStr(i);
      TZQuery(Query).Post;
    end
  else if Query is TSQLQuery then
  begin
    for i := 0 to 99 do
    begin
      TSQLQuery(Query).Append;
      TSQLQuery(Query).FieldByName('str1').AsString := 'first entry ' + IntToStr(i);
      TSQLQuery(Query).FieldByName('str2').AsString := 'second entry ' + IntToStr(i);
      TSQLQuery(Query).FieldByName('str3').AsString := 'third entry ' + IntToStr(i);
      TSQLQuery(Query).Post;
    end;
    TSQLQuery(Query).ApplyUpdates;
    TSQLite3Connection(TSQLQuery(Query).DataBase).Transaction.Commit;
  end
  else
    raise Exception.Create('TForm1.TestFill2: Unknown Query');
end;

end.

